#ifndef DRAW_DIALOG_H_INC
#define DRAW_DIALOG_H_INC

#include <map>
#include <utility>

#include "dstddef.h"
#include <string>
#include "main.h"

class ModalDialogClass
{
public:
   HWND GetWindowHandle() { return WindowHandle; }

protected:
   ModalDialogClass()
   {
   }

   virtual ~ModalDialogClass()
   {
   }
   static UINT DoDialog(void *aObj, HWND aParent, UINT aResourceID)
   {
      if(!AppClass::GetApp()->GetInstanceHandle())
         return 0;
      int st = DialogBoxParam(AppClass::GetApp()->GetInstanceHandle(), MAKEINTRESOURCE(aResourceID), aParent, (DLGPROC )DialogProc, (LPARAM )aObj);
      if(st < 0)
      {
         char Text[1024];
         sprintf(Text, "Dialog error: %d", GetLastError());
         MessageBox(0, Text, "---", MB_OK);
      }
      return st;
   }

   virtual void OnInitDialog() = 0;
   virtual bool OnClose(UINT aMSG) = 0;
   virtual bool OnCommand(UINT aId, UINT aMsg, HWND aCtrlHwnd)
   {
      return TRUE;
   }
   virtual void OnHScroll(uint16_t aCode, uint16_t aPos, HWND aCtrlHWnd)
   {
   }
   virtual void OnTimer(uint32_t aTimer)
   {
   }
   virtual void OnDrawItem(uint32_t aCtrlId, DRAWITEMSTRUCT *aDrawItemStruct)
   {

   }
   virtual void OnSize(uint32_t aType, uint16_t aWidth, uint16_t aHeight)
   {

   }
   virtual bool OnSysCommand(uint32_t aSysCommand)
   {
      return false;
   }
   virtual bool OnMessage(UINT msg, WPARAM wParam, LPARAM lParam)
   {
      return false;
   }

   static UINT DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
   {
      ModalDialogClass *Dialog = 0;
      DialogMapClass::iterator idx = DialogMap.find(hwndDlg);
      if(idx == DialogMap.end())
      {
         if(msg != WM_INITDIALOG)
            return TRUE;
      }
      else
      {
         Dialog = idx->second;
      }
      switch (msg)
      {
      case WM_COMMAND:
         switch (LOWORD(wParam))
         {
         case IDOK:
            if(Dialog->OnClose(IDOK))
            {
               EndDialog(hwndDlg, 1);
               DialogMap.erase(idx);
            }
            return TRUE;
         case IDCANCEL:
            if(Dialog->OnClose(IDCANCEL))
            {
               EndDialog(hwndDlg, 0);
               DialogMap.erase(idx);
            }
            return TRUE;
         default:
            Dialog->OnCommand(LOWORD(wParam), HIWORD(wParam), (HWND )lParam);
         }
         return TRUE;
      case WM_DRAWITEM:
         Dialog->OnDrawItem(wParam, (DRAWITEMSTRUCT *)lParam);
         return TRUE;
      case WM_TIMER:
         Dialog->OnTimer(wParam);
         return TRUE;
      case WM_SIZE:
         Dialog->OnSize(wParam, LOWORD(lParam), HIWORD(lParam));
         return TRUE;
      case WM_SYSCOMMAND:
         return Dialog->OnSysCommand(wParam);
      case WM_INITDIALOG:
      {
         ModalDialogClass *Obj = (ModalDialogClass *)lParam;
         DialogMap.insert(std::make_pair(hwndDlg, Obj));
         Obj->WindowHandle = hwndDlg;
         Obj->OnInitDialog();
         return TRUE;
      }
      case WM_CLOSE:
         if(Dialog->OnClose(WM_CLOSE))
         {
            EndDialog(hwndDlg, 0);
            DialogMap.erase(idx);
         }
         return TRUE;
      case WM_HSCROLL:
         Dialog->OnHScroll(LOWORD(wParam), HIWORD(wParam), (HWND )lParam);
         return TRUE;
      default:
         return Dialog->OnMessage(msg, wParam, lParam);
      }
      return FALSE;
   }
private:
   typedef std::map<HWND, ModalDialogClass *>DialogMapClass;
   static DialogMapClass DialogMap;

protected:
   HWND WindowHandle;

   // Some handy utility functions
   HWND GetDlgItem(int id)
   {  return ::GetDlgItem(WindowHandle, id); }
   LRESULT SendDlgItemMessage(int id, UINT Msg, WPARAM wParam, LPARAM lParam)
   {  return ::SendDlgItemMessage(WindowHandle, id, Msg, wParam, lParam); }
   bool SetDlgItemText(int id, const char *text)
   {  return ::SetDlgItemText(WindowHandle, id, text); }
   bool SetDlgItemText(int id, std::string str)
   {  return ::SetDlgItemText(WindowHandle, id, str.c_str()); }
   bool GetDlgItemText(int id, char *text, size_t Size)
   {  return ::GetDlgItemText(WindowHandle, id, text, Size); }
   bool SetDlgItemInt(int id, uint32_t Value)
   {  return ::SetDlgItemInt(WindowHandle, id, Value, FALSE); }

   LRESULT SendMessage(UINT aMsg, WPARAM aWParam, LPARAM aLParam)
   {
      return ::SendMessage(WindowHandle, aMsg, aWParam, aLParam);
   }
   void StartTimer(uint32_t aId, int32_t aDuration)
   {
      SetTimer(WindowHandle, aId, aDuration, 0);
   }
   void StopTimer(uint32_t aId)
   {
      KillTimer(WindowHandle, aId);
   }

   bool GetDlgItemText(int id, std::string &str)
   {
      HWND Item = GetDlgItem(id);
      if(!Item)
         return FALSE;
      int Length = GetWindowTextLength(Item) + 1;
      char *Temp = new char [Length];

      if(!::GetWindowText(Item, Temp, Length))
      {
         delete [] Temp;
         return FALSE;
      }
      str = Temp;
      delete [] Temp;
      return TRUE;
   }
   bool GetDlgItemInt(int id, uint32_t &Value)
   {
      BOOL Success;
      Value = ::GetDlgItemInt(WindowHandle, id, &Success, FALSE);
      return Success;
   }

   bool GetDlgItemInt(int id, int32_t &Value)
   {
      BOOL Success;
      Value = (int32_t )::GetDlgItemInt(WindowHandle, id, &Success, FALSE);
      return Success;

   }
};

#endif
