#ifndef DRAW_MAIN_H_INC
#define DRAW_MAIN_H_INC

#include <string>
#include <sstream>
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <stdio.h>
#include <map>
#include <vector>

class AppClass
{
public:
   HINSTANCE GetInstanceHandle() {return InstanceHandle; }
   static AppClass *GetApp()
   {
      if(Me == 0)
      {
         MessageBox(0, "No app instance!", "---", MB_OK | MB_ICONWARNING);
      }
      return Me;
   }

   AppClass(HINSTANCE aInstanceHandle)
   {
      InstanceHandle = aInstanceHandle;
      Me = this;
   }
   static AppClass *Me;
   HINSTANCE InstanceHandle;
};

class RegistryClass
{
public:
   RegistryClass(const char *aKey)
   {
      if(RegOpenKey(HKEY_CURRENT_USER, aKey, &KeyHandle) != ERROR_SUCCESS)
      {
         if(RegCreateKeyEx(HKEY_CURRENT_USER, aKey, 0, 0, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, 0, &KeyHandle, 0) != ERROR_SUCCESS)
            KeyHandle = 0;
      }
   }
   ~RegistryClass()
   {
      if(KeyHandle)
         RegCloseKey(KeyHandle);
   }
   int GetInt(const char *aKey, int aDefaultValue = 0)
   {
      if(!KeyHandle)
        return aDefaultValue;

      DWORD Type = REG_DWORD, Size = sizeof(DWORD);
      int Value;
      if(RegQueryValueEx(KeyHandle, aKey, 0, &Type, (BYTE *)&Value, &Size) != ERROR_SUCCESS)
         return aDefaultValue;

      return Value;
   }

   void WriteInt(const char *aKey, int aValue)
   {
      if(!KeyHandle)
         return;
      RegSetValueEx(KeyHandle, aKey, 0, REG_DWORD, (BYTE *)&aValue, sizeof(aValue));
   }

private:
   HKEY KeyHandle;
};

extern AppClass App;

#endif
